#!/usr/bin/env python3

# Copyright 2017-2020 Palantir Technologies, Inc.
# Copyright 2021- Python Language Server Contributors.

from setuptools import setup, find_packages


if __name__ == "__main__":
    setup(
        name="python-lsp-server",  # to allow GitHub dependency tracking work
    )
